#pragma once

#include "caosdb/file_transmission/FileLock.h" // for FileMutex, FileReadLock
#include <boost/filesystem/fstream.hpp>        // for ifstream
#include <boost/filesystem/operations.hpp>     // for exists
#include <boost/filesystem/path.hpp>           // for path
#include <fstream>                             // for ifstream, size_t
#include <memory>                              // for shared_ptr
#include <string>                              // for string

namespace FileExchange {
using boost::filesystem::exists;
using boost::filesystem::ifstream;
using boost::filesystem::path;

class FileReader final {
public:
  FileReader(boost::filesystem::path filename);
  FileReader(boost::filesystem::path filename,
             std::shared_ptr<FileMutex> mutexPtr);

  ~FileReader() = default;

  FileReader(const FileReader &) = delete;
  FileReader &operator=(const FileReader &) = delete;

  FileReader(FileReader &&) = default;
  FileReader &operator=(FileReader &&) = default;

  unsigned long long fileSize() const { return size_; }

  std::size_t read(std::string &buffer);

private:
  void openFile();

  std::ifstream stream_;
  boost::filesystem::path filename_;
  unsigned long long size_;

  std::shared_ptr<FileMutex> mutexPtr_;
  FileReadLock lock_;
};

} // namespace FileExchange
