#include "caosdb/file_transmission/Client.h"
#include "caosdb/file_transmission/DownloadRequestHandler.h" // for DownloadReq...
#include "caosdb/file_transmission/UploadRequestHandler.h" // for UploadReque...
#include "caosdb/logging.h"                                // for CAOSDB_LOG_...
#include "caosdb/status_code.h"                            // for StatusCode
#include <boost/log/core/record.hpp>                       // for record
#include <boost/log/sources/record_ostream.hpp>            // for basic_recor...
#include <boost/preprocessor/seq/limits/enum_256.hpp>      // for BOOST_PP_SE...
#include <boost/preprocessor/seq/limits/size_256.hpp>      // for BOOST_PP_SE...
#include <exception>                                       // IWYU pragma: keep
// IWYU pragma: no_include <bits/exception.h>
#include <grpcpp/impl/codegen/completion_queue.h> // for CompletionQ...

namespace FileExchange {
using caosdb::StatusCode;

FileExchangeClient::~FileExchangeClient() {
  this->cancel();

  cq_.Shutdown();

  // drain the queue
  void *ignoredTag = nullptr;
  bool ok = false;
  while (cq_.Next(&ignoredTag, &ok)) {
    ;
  }
}

StatusCode FileExchangeClient::upload(const FileDescriptor &file_descriptor) {
  handler_ = std::make_unique<UploadRequestHandler>(&handler_, stub_.get(),
                                                    &cq_, file_descriptor);

  int status = this->processMessages();
  if (status > 0) {
    return StatusCode::FILE_UPLOAD_ERROR;
  }
  return StatusCode::SUCCESS;
}

StatusCode FileExchangeClient::download(const FileDescriptor &file_descriptor) {
  handler_ = std::make_unique<DownloadRequestHandler>(&handler_, stub_.get(),
                                                      &cq_, file_descriptor);

  int status = this->processMessages();
  if (status > 0) {
    return StatusCode::FILE_DOWNLOAD_ERROR;
  }
  return StatusCode::SUCCESS;
}

void FileExchangeClient::cancel() {
  if (handler_) {
    handler_->cancel();
  }
}

int FileExchangeClient::processMessages() {
  try {
    void *tag = nullptr;
    bool ok = false;
    while (true) {
      if (cq_.Next(&tag, &ok)) {
        if (tag != nullptr) {
          // TODO(tf): assert
          auto res = handler_->onNext(ok);
          if (!res) {
            // TODO(tf): comment
            handler_.reset();
            break;
          }
        } else {
          CAOSDB_LOG_ERROR(logger_name)
            << "Invalid tag delivered by notification queue.";
        }
      } else {
        CAOSDB_LOG_ERROR(logger_name)
          << "Notification queue has been shut down unexpectedly.";
        return 1;
      }
    }
  } catch (std::exception &e) {
    CAOSDB_LOG_ERROR(logger_name) << "Caught exception: " << e.what();
    return 1;
  } catch (...) {
    CAOSDB_LOG_ERROR(logger_name) << "Caught unknown exception.";
    return 1;
  }
  return 0;
}

} // namespace FileExchange
